--- installpkg.orig	2009-09-06 21:58:16.000000000 +0100
+++ installpkg	2009-09-06 22:03:15.000000000 +0100
@@ -21,6 +21,11 @@
 #  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 #  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
+# Sun Sep  6 21:58:36 BST 2009
+# Replaced usage of "cat" with STDIN redirection or file name parameters
+# to speed up execution on ARM.
+# Replaced pkgbase & package_name code with 'sed' script by Jim Hawkins.
+#
 # Sat Apr 25 21:18:53 UTC 2009
 # Converted to use new pkgbase() function to remove pathname and
 # valid package extensions.
@@ -56,25 +61,8 @@
 # Return a package name that has been stripped of the dirname portion
 # and any of the valid extensions (only):
 pkgbase() {
-  PKGEXT=$(echo $1 | rev | cut -f 1 -d . | rev)
-  case $PKGEXT in
-  'tgz' )
-    PKGRETURN=$(basename $1 .tgz)
-    ;;
-  'tbz' )
-    PKGRETURN=$(basename $1 .tbz)
-    ;;
-  'tlz' )
-    PKGRETURN=$(basename $1 .tlz)
-    ;;
-  'txz' )
-    PKGRETURN=$(basename $1 .txz)
-    ;;
-  *)
-    PKGRETURN=$(basename $1)
-    ;;
-  esac
-  echo $PKGRETURN
+  # basename + strip extensions .tbz, .tgz, .tlz and .txz
+  echo "$1" | sed 's?.*/??;s/\.t[bglx]z$//'
 }
 
 # If installpkg encounters a problem, it will return a non-zero error code.
@@ -142,33 +130,9 @@
   done
 }
 
+# Strip version, architecture and build from the end of the name
 package_name() {
-  STRING=$(pkgbase $1)
-  # Check for old style package name with one segment:
-  if [ "$(echo $STRING | cut -f 1 -d -)" = "$(echo $STRING | cut -f 2 -d -)" ]; then
-    echo $STRING
-  else # has more than one dash delimited segment
-    # Count number of segments:
-    INDEX=1
-    while [ ! "$(echo $STRING | cut -f $INDEX -d -)" = "" ]; do
-      INDEX=$(expr $INDEX + 1)
-    done
-    INDEX=$(expr $INDEX - 1) # don't include the null value
-    # If we don't have four segments, return the old-style (or out of spec) package name:
-    if [ "$INDEX" = "2" -o "$INDEX" = "3" ]; then
-      echo $STRING
-    else # we have four or more segments, so we'll consider this a new-style name:
-      NAME=$(expr $INDEX - 3)
-      NAME="$(echo $STRING | cut -f 1-$NAME -d -)"
-      echo $NAME
-      # cruft for later ;)
-      #VER=$(expr $INDEX - 2)
-      #VER="$(echo $STRING | cut -f $VER -d -)"
-      #ARCH=$(expr $INDEX - 1)
-      #ARCH="$(echo $STRING | cut -f $ARCH -d -)"
-      #BUILD="$(echo $STRING | cut -f $INDEX -d -)"
-    fi
-  fi
+  pkgbase $1 | sed 's?-[^-]*-[^-]*-[^-]*$??'
 }
 
 # Parse options:
@@ -268,8 +232,8 @@
     esac
     ( cd $TMP/scan$$ ; $packagecompression -dc | $TAR xf - install ) < $1 2> /dev/null 
     if [ -r $TMP/scan$$/install/doinst.sh ]; then
-      if cat $TMP/scan$$/install/doinst.sh | grep ' rm -rf ' 1>/dev/null 2>/dev/null ; then
-        cat $TMP/scan$$/install/doinst.sh | grep ' rm -rf ' > $TMP/scan$$/install/delete
+      if grep ' rm -rf ' $TMP/scan$$/install/doinst.sh 1>/dev/null 2>/dev/null ; then
+        grep ' rm -rf ' $TMP/scan$$/install/doinst.sh > $TMP/scan$$/install/delete
         echo "The following locations will be completely WIPED OUT to allow symbolic"
         echo "links to be made. (We're talking 'rm -rf') These locations may be files,"
         echo "or entire directories.  Be sure you've backed up anything at these"
@@ -429,12 +393,12 @@
   fi
 
   # Gather package infomation into a temporary file:
-  cat $DESCRIPTION | grep "^$packagebase:" | cut -f 2- -d : | cut -b2- 1> $TMP/tmpmsg$$ 2> /dev/null
+  grep "^$packagebase:" $DESCRIPTION | cut -f 2- -d : | cut -b2- 1> $TMP/tmpmsg$$ 2> /dev/null
   if [ "$shortname" != "$packagebase" ]; then
-    cat $DESCRIPTION | grep "^$shortname:" | cut -f 2- -d : | cut -b2- 1>> $TMP/tmpmsg$$ 2> /dev/null
+    grep "^$shortname:" $DESCRIPTION | cut -f 2- -d : | cut -b2- 1>> $TMP/tmpmsg$$ 2> /dev/null
   fi
   # Adjust the length here.  This allows a slack-desc to be any size up to 13 lines instead of fixed at 11.
-  LENGTH=$(cat $TMP/tmpmsg$$ | wc -l)
+  LENGTH=$(wc -l < $TMP/tmpmsg$$ )
   while [ $LENGTH -lt 12 ]; do
     echo >> $TMP/tmpmsg$$
     LENGTH=$(expr $LENGTH + 1)
@@ -468,9 +432,9 @@
       echo "Installing package $(basename $package) $PMSG:"
     fi
     echo "PACKAGE DESCRIPTION:"
-    cat $DESCRIPTION | grep "^$packagebase:" | uniq | sed "s/^$packagebase:/#/g"
+    grep "^$packagebase:" $DESCRIPTION | uniq | sed "s/^$packagebase:/#/g"
     if [ "$shortname" != "$packagebase" ]; then
-      cat $DESCRIPTION | grep "^$shortname:" | uniq | sed "s/^$shortname:/#/g"
+      grep "^$shortname:" $DESCRIPTION | uniq | sed "s/^$shortname:/#/g"
     fi
   elif [ "$MODE" = "infobox" ]; then # install infobox package
     dialog --title "Installing package $shortname $PMSG" --infobox "$(cat $TMP/tmpmsg$$)" 0 0
@@ -497,7 +461,7 @@
 
   # Make sure there are no symbolic links sitting in the way of
   # incoming package files:
-  cat $TMP/tmplist$$ | grep -v "/$" | while read file ; do
+    grep -v "/$" $TMP/tmplist$$ | while read file ; do
     if [ -L "$ROOT/$file" ]; then
       rm -f "$ROOT/$file"
     fi
@@ -514,13 +478,13 @@
     echo "PACKAGE MD5SUM: $(md5sum $package | cut -f 1 -d ' ')" >> $ADM_DIR/packages/$shortname
   fi
   echo "PACKAGE DESCRIPTION:" >> $ADM_DIR/packages/$shortname
-  cat $DESCRIPTION | grep "^$packagebase:" >> $ADM_DIR/packages/$shortname 2> /dev/null
+  grep "^$packagebase:" $DESCRIPTION >> $ADM_DIR/packages/$shortname 2> /dev/null
   if [ "$shortname" != "$packagebase" ]; then
-    cat $DESCRIPTION | grep "^$shortname:" >> $ADM_DIR/packages/$shortname 2> /dev/null
+    grep "^$shortname:" $DESCRIPTION >> $ADM_DIR/packages/$shortname 2> /dev/null
   fi
   echo "FILE LIST:" >> $ADM_DIR/packages/$shortname
   ( cd $ROOT/ ; $packagecompression -dc | $TAR -xlUpvf - ) < $package >> $TMP/$shortname 2> /dev/null
-  if [ "$(cat $TMP/$shortname | grep '^\./' | wc -l | tr -d ' ')" = "1" ]; then
+  if [ "$( grep '^\./' $TMP/$shortname | wc -l | tr -d ' ')" = "1" ]; then
     # Good.  We have a package that meets the Slackware spec.
     cat $TMP/$shortname >> $ADM_DIR/packages/$shortname
   else
@@ -542,7 +506,11 @@
     if [ "$MODE" = "install" ]; then
       echo "Executing install script for $(basename $package)."
     fi
-    ( cd $ROOT/ ; sh install/doinst.sh -install; )
+    #    ( cd $ROOT/ ; sh install/doinst.sh -install; )
+    # This tonne of sed is to avoid spawning subshells which is slow on ARM:
+    # the /proc/self stuff is because I couldn't work out how to specify '-install' as an argument
+    # when piping code into the shell.
+    ( cd $ROOT/ ; sed -e's?^( cd \([^;]*\);\(.*\) )$?pushd \1 \&\> /dev/null ; \2 ; popd \&\> /dev/null?g ' install/doinst.sh | sh /proc/self/fd/0 -install ; )
   fi 
   # Clean up the mess...
   if [ -d $ROOT/install ]; then
